@@ -11,10 +11,10 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 from intern.resource import Resource
 from abc import abstractmethod
 
+
 class BossResource(Resource):
     """Base class for Boss resources.
 
@@ -77,20 +77,6 @@ class BossResource(Resource):
         """
         assert NotImplemented
 
-    @abstractmethod
-    def get_reserve_route(self):
-        """Get the route for reserving ids.
-
-        Not all resources will support this operation.
-
-        Returns:
-            (string): A string that can be used as part of a URL.
-
-        Raises:
-            (RuntimeError): if operation not supported by the resource.
-        """
-        assert NotImplemented
-
     @abstractmethod
     def get_meta_route(self):
         """Get the route for metadata operations.
@@ -144,9 +130,6 @@ class CollectionResource(BossResource):
     def get_cutout_route(self):
         raise RuntimeError('Not supported for collections.')
 
-    def get_reserve_route(self):
-        raise RuntimeError('Not supported for collections.')
-
     def get_meta_route(self):
         return self.name
 
@@ -163,33 +146,28 @@ class ExperimentResource(BossResource):
         num_hierarchy_levels (int):
         hierarchy_method (string):
         num_time_samples (int):
-        time_step (int): Defaults to 0.
-        time_step_unit (string): 'nanoseconds', 'microseconds', 'milliseconds', 'seconds'.  Defaults to 'seconds'.
     """
-    def __init__(self, name, collection_name, coord_frame='', description='',
-        num_hierarchy_levels=1, hierarchy_method='anisotropic',
-        num_time_samples=1, creator='', raw={}, 
-        time_step=0, time_step_unit='seconds'):
+    def __init__(self, name, collection_name, coord_frame, description='',
+        num_hierarchy_levels=1, hierarchy_method='near_iso',
+        num_time_samples=1, creator='', raw={}):
         """Constructor.
 
         Args:
             name (string): Experiment name.
             collection_name (string): Collection name.
-            coord_frame (string): Name of coordinate frame used by experiment.  Defaults to empty.
+            coord_frame (string): Name of coordinate frame used by experiment.
             description (optional[string]): Experiment description.  Defaults to empty.
             num_hierarchy_levels (optional[int]): Defaults to 1.
-            hierarchy_method (optional[string]): 'anisotropic', 'isotropic'  Defaults to 'anisotropic'.
+            hierarchy_method (optional[string]): 'near_iso', 'iso', 'slice'  Defaults to 'near_iso'.
             num_time_samples (optional[int]): Maximum number of time samples for any time series data captured by this experiment.  Defaults to 1.
             creator (optional[string]): Resource creator.
             raw (optional[dictionary]): Holds JSON data returned by the Boss API on a POST (create) or GET operation.
-            time_step (optional[int]): Defaults to 0.
-            time_step_unit (optional[string]): 'nanoseconds', 'microseconds', 'milliseconds', 'seconds'.  Defaults to 'seconds'.
         """
 
         BossResource.__init__(self, name, description, creator, raw)
         self.coll_name = collection_name
 
-        self._valid_hierarchy_methods = ['anisotropic', 'isotropic']
+        self._valid_hierarchy_methods = ['near_iso', 'iso', 'slice']
 
         #ToDo: validate data types.
         self._coord_frame = coord_frame
@@ -198,21 +176,9 @@ class ExperimentResource(BossResource):
             hierarchy_method)
         self.num_time_samples = num_time_samples
 
-        self._valid_time_units = [
-            '', 'nanoseconds', 'microseconds', 'milliseconds', 'seconds']
-        self.time_step = time_step
-        self._time_step_unit = self.validate_time_units(time_step_unit)
-
     @property
     def coord_frame(self):
-        if self._coord_frame is '':
-            raise ValueError('Coordinate frame name is blank.')
-        else:
-            return self._coord_frame
-
-    @coord_frame.setter
-    def coord_frame(self, value):
-        self._coord_frame = value
+        return self._coord_frame
 
     @property
     def hierarchy_method(self):
@@ -228,20 +194,6 @@ class ExperimentResource(BossResource):
         """
         self._hierarchy_method = self.validate_hierarchy_method(value)
 
-    @property
-    def time_step_unit(self):
-        return self._time_step_unit
-
-    @time_step_unit.setter
-    def time_step_unit(self, value):
-        """
-        Args:
-            value (string): 'nanoseconds', 'microseconds', 'milliseconds', 'seconds'
-        Raises:
-            ValueError
-        """
-        self._time_step_unit = self.validate_time_units(value)
-
     def get_route(self):
         return self.coll_name + '/experiment/' + self.name
 
@@ -251,9 +203,6 @@ class ExperimentResource(BossResource):
     def get_cutout_route(self):
         raise RuntimeError('Not supported for experiments.')
 
-    def get_reserve_route(self):
-        raise RuntimeError('Not supported for experiments.')
-
     def get_meta_route(self):
         return self.coll_name + '/' + self.name
 
@@ -263,12 +212,6 @@ class ExperimentResource(BossResource):
             return lowered
         raise ValueError('{} is not a valid hierarchy method.'.format(value))
 
-    def validate_time_units(self, value):
-        lowered = value.lower()
-        if lowered in self._valid_time_units:
-            return lowered
-        raise ValueError('{} is not a valid time unit.'.format(value))
-
     def get_dict_route(self):
         return {"collection": self.coll_name, "experiment": self.name}
 
@@ -299,7 +242,7 @@ class CoordinateFrameResource(BossResource):
         self, name, description='',
         x_start=0, x_stop=1, y_start=0, y_stop=1, z_start=0, z_stop=1,
         x_voxel_size=1, y_voxel_size=1, z_voxel_size=1, voxel_unit='nanometers',
-        creator='', raw={}):
+        time_step=0, time_step_unit='seconds', creator='', raw={}):
         """Constructor.
 
         For all ranges, the _stop value is exclusive.  This means valid values will be _less than_ the stop value.
@@ -317,6 +260,8 @@ class CoordinateFrameResource(BossResource):
             y_voxel_size (optional[int]): Defaults to 1.
             z_voxel_size (optional[int]): Defaults to 1.
             voxel_unit (optional[string]): 'nanometers', 'micrometers', 'millimeters', 'centimeters'.  Defaults to 'nanometers'.
+            time_step (optional[int]): Defaults to 0.
+            time_step_unit (optional[string]): 'nanoseconds', 'microseconds', 'milliseconds', 'seconds'.  Defaults to 'seconds'.
             creator (optional[string]): Resource creator.
             raw (optional[dictionary]): Holds JSON data returned by the Boss API on a POST (create) or GET operation.
         """
@@ -326,6 +271,9 @@ class CoordinateFrameResource(BossResource):
         self._valid_voxel_units = [
             'nanometers', 'micrometers', 'millimeters', 'centimeters']
 
+        self._valid_time_units = [
+            'nanoseconds', 'microseconds', 'milliseconds', 'seconds']
+
         self.x_start = x_start
         self.x_stop = x_stop
         self.y_start = y_start
@@ -336,6 +284,8 @@ class CoordinateFrameResource(BossResource):
         self.y_voxel_size = y_voxel_size
         self.z_voxel_size = z_voxel_size
         self._voxel_unit = self.validate_voxel_units(voxel_unit)
+        self.time_step = time_step
+        self._time_step_unit = self.validate_time_units(time_step_unit)
 
     def get_route(self):
         return self.name
@@ -346,9 +296,6 @@ class CoordinateFrameResource(BossResource):
     def get_cutout_route(self):
         raise RuntimeError('Not supported for coordinate frames.')
 
-    def get_reserve_route(self):
-        raise RuntimeError('Not supported for coordinate frames.')
-
     def get_meta_route(self):
         raise RuntimeError('Not supported for coordinate frames.')
 
@@ -368,13 +315,7 @@ class CoordinateFrameResource(BossResource):
 
     @property
     def time_step_unit(self):
-        """
-        Time step unit now part of Experiment.
-
-        Raises:
-            TypeError
-        """
-        raise TypeError('time_step_unit now part of Experiment.')
+        return self._time_step_unit
 
     @time_step_unit.setter
     def time_step_unit(self, value):
@@ -382,9 +323,9 @@ class CoordinateFrameResource(BossResource):
         Args:
             value (string): 'nanoseconds', 'microseconds', 'milliseconds', 'seconds'
         Raises:
-            TypeError
+            ValueError
         """
-        raise TypeError('time_step_unit now part of Experiment.')
+        self._time_step_unit = self.validate_time_units(value)
 
     def validate_voxel_units(self, value):
         lowered = value.lower()
@@ -392,19 +333,16 @@ class CoordinateFrameResource(BossResource):
             return lowered
         raise ValueError('{} is not a valid voxel unit.'.format(value))
 
+    def validate_time_units(self, value):
+        lowered = value.lower()
+        if lowered in self._valid_time_units:
+            return lowered
+        raise ValueError('{} is not a valid time unit.'.format(value))
 
     def get_dict_route(self):
         return {"coord": self.name}
 
 
-class PartialChannelResourceError(Exception):
-    """
-    Indicate that the ChannelResource is not fully initialized for use with
-    the cutout service.
-    """
-    pass
-
-
 class ChannelResource(BossResource):
     """
     Holds channel data.
@@ -418,26 +356,16 @@ class ChannelResource(BossResource):
         _type (string): 'image' or 'annotation'
         _valid_datatypes (list[string]): Allowed data type values (static variable).
         _datatype (string):
-        _cutout_ready (bool): True if datatype specified during instantiation.
     """
 
     _valid_datatypes = ['uint8', 'uint16', 'uint64']
     _valid_types = ['annotation', 'image']
-    _valid_downsample_status = ['NOT_DOWNSAMPLED', 'IN_PROGRESS', 'DOWNSAMPLED', "FAILED"]
 
     def __init__(self, name, collection_name, experiment_name, type="image",
-        description='', default_time_sample=0, datatype='',
-        base_resolution=0, sources=[], related=[], creator='', downsample_status="NOT_DOWNSAMPLED", raw={}):
+        description='', default_time_sample=0, datatype='uint8',
+        base_resolution=0, sources=[], related=[], creator='', raw={}):
         """Constructor.
 
-        Note, if the channel _already_ exists in the Boss, you can use the
-        helper method get_channel(), that is part of BossRemote, to 
-        instantiate the ChannelResource with all the correct values, using only
-        the names of the collection, experiment, and channel.  
-
-        rmt = BossRemote()
-        channel = rmt.get_channel('myChannel', 'myCollection', 'myExperiment')
-
         Args:
             name (string): Channel name.
             collection_name (string): Parent collection name.
@@ -450,7 +378,6 @@ class ChannelResource(BossResource):
             sources (optional[list[string]]): Channels this channel was derived from.
             related (optiona[list[string]]): Channels related to this channel.
             creator (optional[string]): Resource creator.
-            downsample_status (str): Status indicating if the channel has been downsampled or not
             raw (optional[dictionary]): Holds JSON data returned by the Boss API on a POST (create) or GET operation.
         """
 
@@ -459,15 +386,6 @@ class ChannelResource(BossResource):
         self.exp_name = experiment_name
 
         self._type = self.validate_type(type)
-
-        # Class is considered fully initialized if datatype set during
-        # initialization.
-        if datatype == '':
-            self._cutout_ready = False
-            # Default to uint8.
-            datatype = 'uint8'
-        else:
-            self._cutout_ready = True
         self._datatype = self.validate_datatype(datatype)
 
         self.sources = sources
@@ -477,7 +395,6 @@ class ChannelResource(BossResource):
         #ToDo: validate data types.
         self.default_time_sample = default_time_sample
         self.base_resolution = base_resolution
-        self.downsample_status = self.validate_downsample_status(downsample_status)
 
     def get_route(self):
         return self.coll_name + '/experiment/' + self.exp_name + '/channel/' + self.name
@@ -488,9 +405,6 @@ class ChannelResource(BossResource):
     def get_cutout_route(self):
         return self.coll_name + '/' + self.exp_name + '/' + self.name
 
-    def get_reserve_route(self):
-        return self.coll_name + '/' + self.exp_name + '/' + self.name
-
     def get_meta_route(self):
         return self.coll_name + '/' + self.exp_name + '/' + self.name
 
@@ -499,15 +413,6 @@ class ChannelResource(BossResource):
         """
         return True
 
-    @property
-    def cutout_ready(self):
-        """Is this channel fully configured for doing cutouts?
-
-        Returns:
-            (bool) True if the datatype was explicitly set by the user.
-        """
-        return self._cutout_ready
-
     @property
     def sources(self):
         return self._sources
@@ -567,7 +472,6 @@ class ChannelResource(BossResource):
             ValueError
         """
         self._datatype = self.validate_datatype(value)
-        self._cutout_ready = True
 
     def validate_type(self, value):
         lowered = value.lower()
@@ -581,11 +485,5 @@ class ChannelResource(BossResource):
             return lowered
         raise ValueError('{} is not a valid data type.'.format(value))
 
-    def validate_downsample_status(self, value):
-        lowered = value.upper()
-        if lowered in ChannelResource._valid_downsample_status:
-            return lowered
-        raise ValueError('{} is not a valid downsample status.'.format(value))
-
     def get_dict_route(self):
         return {"collection": self.coll_name, "experiment": self.exp_name, "channel": self.name}
